
/**
 * Module dependencies.
 */

var rfc2445 = require('../lib/rfc2445')
  , Util = require('../lib/util')
  , assert = require('assert')

var _print = require('./print');

/**
 * Alias
 */

var _date = Util.date
  , _str = Util.string;

// testing vars

/**
 * daily recurrence
 * - interval: 3
 * - ends:
 *    - after: 5
 *
 * use TZID format fot start and end
 */
var RFC2445_rec_daily_after =
      'DTSTART;TZID=America/Argentina/Buenos_Aires:20110901T100000\r\n'
    + 'DTEND;TZID=America/Argentina/Buenos_Aires:20110902T110000\r\n'
    + 'RRULE:FREQ=DAILY;COUNT=5;INTERVAL=3';

var recObjDailyAfter = {
        description: ''
      , location: 'America/Argentina/Buenos_Aires'
      , start: new Date(2011, 08, 1, 10, 0, 0)
      , end: new Date(2011, 8, 02, 11, 0, 0, 0)
      , created: new Date(2011, 09, 5, 19, 55, 20, 0)
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'All Day'
      , updated: new Date()
      , recurrence: {
            every: 3
          , 'end-by': {
                type: 'after'
              , after: 5
            }
        }
      , frequency: 'day'
      , allDay: false
    }

/**
 * daily recurrence
 * - interval: 4
 * - ends
 *    - on: 20111028
 */

var RFC2445_rec_daily_on =
        'DTSTART;VALUE=DATE:20110930\r\n'
      + 'DTEND;VALUE=DATE:20111002\r\n'
      + 'RRULE:FREQ=DAILY;UNTIL=20111028;INTERVAL=4';

var recObjDailyOn = {
        description: ''
      , location: 'America/Argentina/Buenos_Aires'
      , start: new Date(2011, 08, 30, 10, 0, 0)
      , end: new Date(2011, 9, 02, 11, 0, 0, 0)
      , created: new Date(2011, 09, 5, 19, 55, 20, 0)
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'All Day'
      , updated: new Date()
      , recurrence: {
            every: 4
          , 'end-by': {
                type: 'on'
              , on: new Date('2011-10-28')
            }
        }
      , frequency: 'day'
      , allDay: true
    }

/**
 * weekly recurrence
 * - repeat-on: {m: 'on', t: 'on', sa: 'on'}
 */

var RFC2445_rec_weekly =
        'DTSTART;VALUE=DATE:20111001\r\n'
      + 'DTEND;VALUE=DATE:20111002\r\n'
      + 'RRULE:FREQ=WEEKLY;BYDAY=MO,TU,SA';

var recObjWeekly = {
        description: ''
      , location: 'America/Argentina/Buenos_Aires'
      , start: new Date(2011, 9, 1, 10, 0, 0)
      , end: new Date(2011, 9, 02, 11, 0, 0, 0)
      , created: new Date(2011, 09, 5, 19, 55, 20, 0)
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'All Day'
      , updated: new Date()
      , recurrence: {
            every: 1
          , 'end-by': {
                type: 'never'
              , on: null
              , after: null
            }
          , 'repeat-on': {
              m: 'on'
            , t: 'on'
            , sa: 'on'
          }
        }
      , frequency: 'week'
      , allDay: true
    }

/**
 * Monthly recurrence - day on month
 */

var RFC2445_rec_monthly_bymonth =
        'DTSTART;VALUE=DATE:20111001\r\n'
      + 'DTEND;VALUE=DATE:20111002\r\n'
      + 'RRULE:FREQ=MONTHLY;COUNT=10;BYMONTHDAY=1';

var recObjMonthlyByMonth = {
        description: ''
      , location: 'America/Argentina/Buenos_Aires'
      , start: new Date(2011, 9, 1, 10, 0, 0)
      , end: new Date(2011, 9, 02, 11, 0, 0, 0)
      , created: new Date(2011, 09, 5, 19, 55, 20, 0)
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'All Day'
      , updated: new Date()
      , recurrence: {
            every: 1
          , 'end-by': {
                type: 'after'
              , on: null
              , after: 10
            }
          , 'repeat-on': {
              mode: 'day-of-month'
            }
        }
      , frequency: 'month'
      , allDay: true
    }

/**
 * Monthly recurrence - day on week
 */

var RFC2445_rec_monthly_byweek = 
        'DTSTART;VALUE=DATE:20111001\r\n'
      + 'DTEND;VALUE=DATE:20111002\r\n'
      + 'RRULE:FREQ=MONTHLY;COUNT=10;BYDAY=1SA';

var recObjMonthlyByWeek = {
        description: ''
      , location: 'America/Argentina/Buenos_Aires'
      , start: new Date(2011, 9, 1, 10, 0, 0)
      , end: new Date(2011, 9, 02, 11, 0, 0, 0)
      , created: new Date(2011, 09, 5, 19, 55, 20, 0)
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'All Day'
      , updated: new Date()
      , recurrence: {
            every: 1
          , 'end-by': {
                type: 'after'
              , on: null
              , after: 10
            }
          , 'repeat-on': {
              mode: 'day-of-week'
            }
        }
      , frequency: 'month'
      , allDay: true
    }

/**
 * Monthly recurrence - day on week
 */

var RFC2445_rec_yearly =
        'DTSTART;VALUE=DATE:20111001\r\n'
      + 'DTEND;VALUE=DATE:20111002\r\n'
      + 'RRULE:FREQ=YEARLY;COUNT=2';

var recObjYearly = {
        description: ''
      , location: 'America/Argentina/Buenos_Aires'
      , start: new Date(2011, 9, 1, 10, 0, 0)
      , end: new Date(2011, 9, 02, 11, 0, 0, 0)
      , created: new Date(2011, 09, 5, 19, 55, 20, 0)
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'All Day'
      , updated: new Date()
      , recurrence: {
            every: 1
          , 'end-by': {
                type: 'after'
              , on: null
              , after: 2
            }
        }
      , frequency: 'year'
      , allDay: true
    }

// start date: Sat, 01 Oct 2011 13:30:00 GMT
var start = new Date(2011, 09, 1, 13, 30, 0)
// end date: Tue, 04 Oct 2011 20:40:00 GMT
  , end = new Date(2011, 09, 4, 20, 40, 0, 0)
  , updated = new Date(2011, 09, 5, 19, 55, 20, 0)

var objNormalEvent = {
        escription: ''
      , location: 'America/Argentina/Buenos_Aires'
      , _calendar: '4e8c6fdb7400eb9014000811'
      , _id: '4e8c6fdb7400eb9014000816'
      , _user: '4e4c2518e7a7cb911c000769'
      , created: 'Wed, 05 Oct 2011 14:55:23 GMT'
      , end: end
      , googleEtag: "E0gIRwZJeCp7JGA6WhJW"
      , googleId: '6cd3g8uhgave9gh2pm2g4d6bo4'
      , googleSelfLink: 'https://www.google.com/calendar/feeds/rdsuarez%40gmail.com/private/full/6cd3g8uhgave9gh2pm2g4d6bo4'
      , start: start
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'Normal Event'
      , updated: new Date()
      , google: true
      , allowRemoveTwins: true
      , recurrence: {
            exceptions: []
          , 'end-by': {
                type: 'never'
              , after: 1
            }
          , every: 1
        }
      , frequency: ''
      , allDay: false
    }


var objRecEvent = {
        escription: ''
      , location: 'America/Argentina/Buenos_Aires'
      , _calendar: '4e8c6fdb7400eb9014000811'
      , _id: '4e8c6fdb7400eb9014000816'
      , _user: '4e4c2518e7a7cb911c000769'
      , created: 'Wed, 05 Oct 2011 14:55:23 GMT'
      , end: end
      , googleEtag: "E0gIRwZJeCp7JGA6WhJW"
      , googleId: '6cd3g8uhgave9gh2pm2g4d6bo4'
      , googleSelfLink: 'https://www.google.com/calendar/feeds/rdsuarez%40gmail.com/private/full/6cd3g8uhgave9gh2pm2g4d6bo4'
      , start: start
      , stop: 'Sun, 02 Oct 2011 03:00:00 GMT'
      , title: 'Recurrence Event'
      , updated: new Date()
      , google: true
      , allowRemoveTwins: true
      , recurrence: {
            exceptions: []
          , 'end-by': {
                type: 'never'
              , after: 1
            }
          , every: 1
        }
      , frequency: 'day'
      , allDay: false
    }

var xml_normal_event = '<entry xmlns="http://www.w3.org/2005/Atom" xmlns:gd="http://schemas.google.com/g/2005" xmlns:gCal="http://schemas.google.com/gCal/2005" gd:fields="gd:recurrence,gd:when"><title>Normal Event</title><gd:when startTime="2011-10-01T13:30:00.000Z" endTime="2011-10-04T20:40:00.000Z" /><gd:where valueString="America/Argentina/Buenos_Aires"></gd:where><updated>2011-10-05T19:55:20.000Z</updated></entry>';



var xml_normal_all_day_event = "<entry gd:etag='&quot;E0kOQARBeSp7JGA6WhJW&quot;'><id>http://www.google.com/calendar/feeds/2qndnag6na7d2oau8oq1gho7h0%40group.calendar.google.com/events/shkf9caob7r7vja5vev2o45rmc_20110901T130000Z</id><title>Rec</title><content/><link rel='self' type='application/atom+xml' href='https://www.google.com/calendar/feeds/2qndnag6na7d2oau8oq1gho7h0%40group.calendar.google.com/private/full/shkf9caob7r7vja5vev2o45rmc_20110901T130000Z'/><gd:where valueString=''/><gd:when endTime='2011-09-30T11:00:00.000-03:00' startTime='2011-09-29T10:00:00.000-03:00'/></entry>"

/**
 * compare properties of two objects
 */

var compareObjects = function (obj1, obj2) {
  var isOk = true;

  for (var k in obj1) {
    if (typeof obj1[k] != 'object') {
      isOk = obj1[k] == obj2[k] && isOk;
      if (!isOk)
        _print('error [' + k +']', obj1[k] + ' != ' + obj2[k], 'E', 'LIGHT_RED')
    }
    else {
      for (var l in obj1[k]) {
        var v1 = obj1[k][l]
          , v2 = obj2[k][l];

        v1 = obj1[k][l] instanceof Date ? +v1 : v1;
        v2 = obj2[k][l] instanceof Date ? +v2 : v2;

        isOk = v1 == v2 && isOk;
        if (!isOk)
          _print('error [' + l + ']', obj1[k][l] + ' != ' + obj2[k][l], 'E', 'LIGHT_RED')

      }
    }
  }

  return isOk;
}

/**
 * dump function to compare two xml string
 */

var compareXMLString = function (str1, str2, cut) {
  cut = cut || '\r\n';

  var arrStr1 = str1.split(cut)
    , arrStr2 = str2.split(cut);

  // length control
  if (arrStr1.length != arrStr2.length) {
    _print('error', 'array by split - length error', 'E', 'LIGHT_RED');
    return false;
  }

  var isOk = true;

  // line by line control
  for (var i = 0; i < arrStr1.length; i++) {
    var row1 = arrStr1[i]
      , row2 = arrStr2[i];

    isOk = (row1 == row2 || row1.match(/<\/updated/)) && isOk;

    if (!isOk) {
      _print('error', 'rows number ' + i + ' are not equal', 'E', 'LIGHT_RED');
      _print('', 'row1: ' + row1, '', 'RED');
      _print('', 'row2: ' + row2, '', 'RED');
      return false;
    }
    
  };

  return isOk;
};

module.exports = {

  /**
   * RFC2445 tests
   */

  // recurrence XML string to event object

  'getFrequency() - DAILY': function () {
    _print('RFC2445', 'getFrequency() - DAILY ')
    assert.equal(rfc2445.getFrequency(RFC2445_rec_daily_after), 'day');
  },

  'getFrequency() - WEEKLY': function () {
    _print('RFC2445', 'getFrequency() - WEEKLY')
    assert.equal(rfc2445.getFrequency(RFC2445_rec_weekly), 'week');
  },

  'getFrequency() - MONTHLY': function () {
    _print('RFC2445', 'getFrequency() - MONTHLY')
    assert.equal(rfc2445.getFrequency(RFC2445_rec_monthly_bymonth), 'month');
  },

  'getFrequency() - YEALRY': function () {
    _print('RFC2445', 'getFrequency() - YEARLY')
    assert.equal(rfc2445.getFrequency(RFC2445_rec_yearly), 'year');
  },

  'getAllDay() - expected all-day=false': function () {
    _print('RFC2445', 'getAllDay() - expected all-day=false');
    assert.ok(!rfc2445.getAllday(RFC2445_rec_daily_after), 'all-day is TRUE');
  },

  'getAllDay() - expected all-day=true': function () {
    _print('RFC2445', 'getAllDay() - expected all-day=true');
    assert.ok(rfc2445.getAllday(RFC2445_rec_daily_on), 'all-day not is TRUE');
  },

  'getRecurrenceObj() - DAILY - after': function (str) {
    _print('RFC2445', 'getRecurrenceObj() - DAILY - after');

    var recObj = rfc2445.getRecurrenceObj(RFC2445_rec_daily_after)
      , isOk = compareObjects(recObjDailyAfter.recurrence, recObj);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'getRecurrenceObj() - DAILY - on': function (str) {
    _print('RFC2445', 'getRecurrenceObj() - DAILY - on')

    var recObj = rfc2445.getRecurrenceObj(RFC2445_rec_daily_on)
      , isOk = compareObjects(recObjDailyOn.recurrence, recObj);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'getRecurrenceObj() - WEEKLY': function (str) {
    _print('RFC2445', 'getRecurrenceObj() - WEEKLY');

    var recObj = rfc2445.getRecurrenceObj(RFC2445_rec_weekly)
      , isOk = compareObjects(recObjWeekly.recurrence, recObj);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'getRecurrenceObj() - MONTHLY - day on month': function (str) {
    _print('RFC2445', 'getRecurrenceObj() - MONTHLY - day on month');

    var recObj = rfc2445.getRecurrenceObj(RFC2445_rec_monthly_bymonth)
      , isOk = compareObjects(recObjMonthlyByMonth.recurrence, recObj);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'getRecurrenceObj() - MONTHLY - day on week': function (str) {
    _print('RFC2445', 'getRecurrenceObj() - MONTHLY - day on week');

    var recObj = rfc2445.getRecurrenceObj(RFC2445_rec_monthly_byweek)
      , isOk = compareObjects(recObjMonthlyByWeek.recurrence, recObj);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'getRecurrenceXML() - YEARLY': function (str) {
    _print('RFC2445', 'getRecurrenceXML() - YEARLY');

    var recObj = rfc2445.getRecurrenceObj(RFC2445_rec_yearly)
      , isOk = compareObjects(recObjYearly.recurrence, recObj);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'genRecurrenceString() - DAILY - after': function (str) {
    _print('RFC2445', 'genRecurrenceString - DAILY - after');

    var string = rfc2445.genRecurrenceString(recObjDailyAfter)
      , isOk = compareXMLString(RFC2445_rec_daily_after, string);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'genRecurrenceString() - DAILY - on': function (str) {
    _print('RFC2445', 'genRecurrenceString() - DAILY - on');

    var string = rfc2445.genRecurrenceString(recObjDailyOn)
      , isOk = compareXMLString(RFC2445_rec_daily_on, string);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'genRecurrenceString() - WEEKLY': function (str) {
    _print('RFC2445', 'genRecurrenceString() - WEEKLY');

    var string = rfc2445.genRecurrenceString(recObjWeekly)
      , isOk = compareXMLString(RFC2445_rec_weekly, string);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'genRecurrenceString() - MONTHLY - day on month': function (str) {
    _print('RFC2445', 'genRecurrenceString() - MONTHLY - day on month');

    var string = rfc2445.genRecurrenceString(recObjMonthlyByMonth)
      , isOk = compareXMLString(RFC2445_rec_monthly_bymonth, string);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'genRecurrenceString() - MONTHLY - day on week': function (str) {
    _print('RFC2445', 'genRecurrenceString() - MONTHLY - day on week');

    var string = rfc2445.genRecurrenceString(recObjMonthlyByWeek)
      , isOk = compareXMLString(RFC2445_rec_monthly_byweek, string);

    assert.ok(isOk, 'some recurrence property have an error');
  },

  'genRecurrenceString() - YEARLY': function (str) {
    _print('RFC2445', 'genRecurrenceString() - YEARLY');

    var string = rfc2445.genRecurrenceString(recObjYearly)
      , isOk = compareXMLString(RFC2445_rec_yearly, string);

    assert.ok(isOk, 'some recurrence property have an error');
  },

};
