
/**
 * Module dependencies.
 */

var Carilu = require('../')
  , assert = require('assert');

var _print = require('./print');

  var mainSeedEv = {
      title: 'Event Instances'
    , allDay: false
    , frequency: 'day'
    , recurrence: {
          'end-by': {
              after: 4
            , type: 'after'
          }
        , every: 4
        , exceptions: []
      }
  }

  var start = new Date(2011, 8, 26, 15, 0, 0)
    , end = new Date(2011, 8, 29, 16, 0, 0);

  function getSeedEvent (_start, _end) {
    var newEvent = JSON.parse(JSON.stringify(mainSeedEv));

    newEvent.start = new Date(_start || start);
    newEvent.end = new Date(_end || end);
    return newEvent;
  }

  var normalEvent = getSeedEvent();
  normalEvent.frequency = '';
  normalEvent.start = new Date(2011, 8, 26, 10, 0, 0);
  normalEvent.end = new Date(2011, 8, 27, 11, 0, 0);

  var seedDailyEvent = getSeedEvent();
  seedDailyEvent.start = new Date(2011, 8, 26, 10);
  seedDailyEvent.end = new Date(2011, 8, 27, 11);
  seedDailyEvent.recurrence.every = 5;

  var seedWeeklyEvent = getSeedEvent();
  seedWeeklyEvent.start = new Date(2011, 8, 30, 15, 0, 0);
  seedWeeklyEvent.end = new Date(2011, 9, 1, 16, 0, 0);
  seedWeeklyEvent.frequency = 'week';
  seedWeeklyEvent.recurrence.every = 2;
  seedWeeklyEvent.recurrence['repeat-on'] = { f: 'on', t: 'on', m: 'on' };

  var seedMonthlyEvent = getSeedEvent();
  seedMonthlyEvent.frequency = 'month';
  seedMonthlyEvent.recurrence.every = 2;

  var seedYearlyEvent = getSeedEvent();
  seedYearlyEvent.frequency = 'year';
  seedYearlyEvent.start = new Date(2011, 8, 26, 15);
  seedYearlyEvent.end = new Date(2011, 8, 29, 16);
  seedYearlyEvent.recurrence.every = 2;

    /**
     * Oct 2011
     *
     *   S    M    T    W   Th    F   Sa
     *  ---------------------------------
     *  25   26   27   28   29   30    1
     *   2    3    4    5    6    7    8
     *   9   10   11   12   13   14   15
     *  16   17   18   19   20   21   22
     *  23   24   25   26   27   28   29
     *  30   31    1    2    3    4    5
     */

  module.exports = {
  '_getDatePrevInstance() - day frequency': function() {
    _print ('Seed\t', '_getDatePrevInstance() - day');

    // seed object
    var Seed = Carilu.seed(seedDailyEvent);

    // prev date into current period
    var referenceDate = new Date(2011, 8, 24);
    var prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 10:00:00 GMT', prevInstance.toGMTString());

    // retrieving previous date passing reference date
    referenceDate = new Date(2011, 8, 30, 10, 1, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 1, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 2, 11, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 2, 23, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 5, 17, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 5, 17, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 05 Nov 2011 10:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - day frequency - allDay': function() {
    _print ('Seed\t', '_getDatePrevInstance() - day - allDay');

    seedDailyEvent.allDay = true;
    var Seed = Carilu.seed(seedDailyEvent);

    // prev date into current period
    var referenceDate = new Date(2011, 8, 24);
    var prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 0, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 11, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 2, 23, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 4, 23, 59, 59, 999);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 6, 0, 0, 1);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Thu, 06 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - week frequency': function() {
    _print ('Seed\t', '_getDatePrevInstance() - week');

   // seed object
    var Seed = Carilu.seed(seedWeeklyEvent);

    var referenceDate = new Date(2011, 8, 25);
    var prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Fri, 30 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 17, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Fri, 30 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 5, 17, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Fri, 30 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 10, 17, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 10 Oct 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 11, 5, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 10 Oct 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 11, 22, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 11 Oct 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 13, 22, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 11 Oct 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 6, 5, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 05 Dec 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 6, 16, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 06 Dec 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 8, 20, 0, 0);
    nextInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Tue, 06 Dec 2011 15:00:00 GMT', nextInstance.toGMTString());

  },
  '_getDatePrevInstance() - monthly frequency - day of month': function() {
    _print ('Seed\t', '_getDatePrevInstance() - monthly - day of month');

    // set as day-of-month
    seedMonthlyEvent.recurrence['repeat-on'] = { mode: 'day-of-month' };

    // seed object
    var Seed = Carilu.seed(seedMonthlyEvent);

    var referenceDate = new Date(2011, 8, 29, 10);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 10);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 26, 15);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 30, 10);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 10, 12, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());
 
    referenceDate = new Date(2011, 10, 28, 16, 0, 1);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 26, 14, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 26, 15, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 10, 15, 0, 0, 1);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2012, 4, 27);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Sat, 26 May 2012 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDatePrevInstance() - monthly frequency - day of week': function() {
    _print ('Seed\t', '_getDatePrevInstance() - monthly - day of week');

    seedMonthlyEvent.recurrence['repeat-on'] = { mode: 'day-of-week' };

    // seed object
    var Seed = Carilu.seed(seedMonthlyEvent);

    var referenceDate = new Date(2011, 8, 25);
    var prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 8, 29, 17, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 12, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 10, 12, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 26, 16, 0, 1);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 28, 14, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 29, 15, 0, 0, 1);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2012, 4, 27);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Mar 2012 15:00:00 GMT', prevInstance.toGMTString());
  
  },

  '_getDatePrevInstance() - yearly': function() {
    _print ('Seed\t', '_getDatePrevInstance() - yearly');

    var Seed = Carilu.seed(seedYearlyEvent);

    var referenceDate = new Date(2013, 8, 26);
    var prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2013, 8, 26, 14, 59, 59, 59);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2013, 8, 26, 15, 0, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Thu, 26 Sep 2013 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2013, 8, 30, 15, 0, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Thu, 26 Sep 2013 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2014, 8, 26, 15, 0, 0, 0);
    prevInstance = Seed._getDatePrevInstance(referenceDate);
    assert.equal('Thu, 26 Sep 2013 15:00:00 GMT', prevInstance.toGMTString());

  },

  '_getDateNextInstance() - day frequency': function() {
    _print ('Seed\t', '_getDateNextInstance() - day');

    // seed object
    var Seed = Carilu.seed(seedDailyEvent);

    // prev date into current period
    var referenceDate = new Date(2011, 8, 25);
    var prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 10:00:00 GMT', prevInstance.toGMTString());

    // retrieving previous date passing reference date
    referenceDate = new Date(2011, 8, 30, 17, 25, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 1, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 23, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 5, 17, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 06 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 5, 17, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 05 Nov 2011 10:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - day frequency - allDay': function() {
    _print ('Seed\t', '_getDateNextInstance() - day - allDay');

    seedDailyEvent.allDay = true;
    var Seed = Carilu.seed(seedDailyEvent);

    // prev date into current period
    var referenceDate = new Date(2011, 8, 24);
    var prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 0, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 11, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 2, 23, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 01 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 4, 23, 59, 59, 999);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 06 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 6, 0, 0, 1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 06 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 7, 20, 59, 59, 999);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 06 Oct 2011 10:00:00 GMT', prevInstance.toGMTString());


  },


  '_getDateNextInstance() - week frequency': function() {
    _print ('Seed\t', '_getDateNextInstance() - week');

    // seed object
    var Seed = Carilu.seed(seedWeeklyEvent);

    var referenceDate = new Date(2011, 8, 25);
    var nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Fri, 30 Sep 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 12, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Fri, 30 Sep 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 9, 5, 17, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 10 Oct 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 9, 11, 10, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 10 Oct 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 9, 11, 16, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 11 Oct 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 9, 13, 22, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Fri, 14 Oct 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 11, 6, 5, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 05 Dec 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 11, 6, 16, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Tue, 06 Dec 2011 15:00:00 GMT', nextInstance.toGMTString());

    referenceDate = new Date(2011, 11, 8, 20, 0, 0);
    nextInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Fri, 09 Dec 2011 15:00:00 GMT', nextInstance.toGMTString());

  },

  '_getDateNextInstance() - monthly frequency - day of month': function() {
    _print ('Seed\t', '_getDateNextInstance() - monthly - day of month');

    seedMonthlyEvent.recurrence['repeat-on'] = { mode: 'day-of-month' };
    var Seed = Carilu.seed(seedMonthlyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var referenceDate = new Date(2011, 8, 29, 10);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 10);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 29, 10);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 30, 10);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 26 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    var referenceDate = new Date(2011, 8, 25);
    var prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 8, 29, 15, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 12, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 9, 14, 0, 1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 26, 14, 0, 1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 30, 10, 0, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 26 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 30);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 26 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2012, 4, 27);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 May 2012 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - monthly frequency - day of week': function() {
    _print ('Seed\t', '_getDateNextInstance() - monthly - day of week');

    seedMonthlyEvent.recurrence['repeat-on'] = { mode: 'day-of-week' };
    var Seed = Carilu.seed(seedMonthlyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var referenceDate = new Date(208, 8, 27);
    var prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 8, 29, 17, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 9, 1, 12, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 30, 6, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 6, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 16, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 23 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 10, 12, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());
 
    referenceDate = new Date(2011, 10, 26, 16, 0, 1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 28, 14, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 10, 28, 15, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 6, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 15, 59, 59);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 16, 0, 0, -1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 Nov 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 16, 0, 0, 1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 23 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 1, 22, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 23 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 11, 10, 15, 0, 0, 1);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 23 Jan 2012 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2012, 4, 27);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 28 May 2012 15:00:00 GMT', prevInstance.toGMTString());
  },

  '_getDateNextInstance() - yearly': function() {
    _print ('Seed\t', '_getDateNextInstance() - yearly');

    // seed object
    var Seed = Carilu.seed(seedYearlyEvent);

    var referenceDate = new Date(2011, 8, 25);
    var prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Mon, 26 Sep 2011 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2011, 8, 30, 14, 59, 59, 59);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 26 Sep 2013 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2013, 8, 26, 15, 0, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Thu, 26 Sep 2013 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2013, 8, 30, 15, 0, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Sep 2015 15:00:00 GMT', prevInstance.toGMTString());

    referenceDate = new Date(2014, 8, 26, 15, 0, 0, 0);
    prevInstance = Seed._getDateNextInstance(referenceDate);
    assert.equal('Sat, 26 Sep 2015 15:00:00 GMT', prevInstance.toGMTString());

  },

  'getElapsedInstances() - daily': function() {
    _print ('Seed\t', 'getElapsedInstances() - daily');

    // start: 9/26/2011 10:00:00
    // end:   9/27/2011 11:00:00
    var start = new Date(2011, 8, 26, 10, 0, 0, 0)
      , end = new Date(2011, 8, 27, 11, 0, 0, 0);

    seedDailyEvent.allDay = false;
    var Seed = Carilu.seed(seedDailyEvent, {
            start: start
          , end: end
        });

    // limit date into first instances
    Seed.setPeriod(new Date(2011, 8, 27, 11, 0, 0));
    var elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    // limite date less that event start date
    Seed.setPeriod(new Date(2011, 8, 26, 5, 12, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    // limit date into empty date
    Seed.setPeriod(new Date(2011, 8, 30, 20, 30, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    // limit date equal to second instance end date
    Seed.setPeriod(new Date(2011, 9, 1, 17));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    // limit date equal to second instance end date
    Seed.setPeriod(new Date(2011, 9, 2, 11, 0, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 6, 19, 0, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    // limit date into third event instance
    Seed.setPeriod(new Date(2011, 9, 7, 10, 10, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 8, 22, 10, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(3, elapsedInstances);

    Seed.setPeriod(new Date(2011, 1, 1, 15, 59, 59));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 30, 15, 59, 59));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(7, elapsedInstances);

    seedDailyEvent.allDay = true;
    var Seed = Carilu.seed(seedDailyEvent, {
            start: start
          , end: end
        });

    // limit date into first instances
    Seed.setPeriod(new Date(2011, 8, 27, 11, 0, 0));
    var elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    // limite date less that event start date
    Seed.setPeriod(new Date(2011, 8, 26, 5, 12, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    // limit date into empty date
    Seed.setPeriod(new Date(2011, 8, 30, 20, 30, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    // limit date equal to second instance end date
    Seed.setPeriod(new Date(2011, 9, 1, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);


  },

  'getElapsedInstances() - daily - allDay': function() {
    _print ('Seed\t', 'getElapsedInstances() - daily - allDay');

    seedDailyEvent.allDay = true;
    var Seed = Carilu.seed(seedDailyEvent, {
            start: new Date(2011, 8, 26, 10)
          , end: new Date(2011, 8, 27, 11)
        });

    // limit date into first instances
    Seed.setPeriod(new Date(2011, 8, 26, 12, 10, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    // limit date into first instances (add partial instance)
    Seed.setPeriod(new Date(2011, 8, 26, 12, 10, 0));
    elapsedInstances = Seed.getElapsedInstances(null, true);
    assert.equal(0, elapsedInstances);

    // limite date less that event start date
    Seed.setPeriod(new Date(2011, 8, 27, 22, 12, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

/** // limite date less that event start date
    Seed.setPeriod(new Date(2011, 8, 28));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);
*/
    // limite date less that event start date
    Seed.setPeriod(new Date(2011, 9, 7, 23, 0, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    // limite date less that event start date
    Seed.setPeriod(new Date(2011, 9, 8, 0, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(3, elapsedInstances);
  },

  'getElapsedInstances() - weekly': function() {
    _print ('Seed\t', 'getElapsedInstances() - weekly');

    var start = new Date(2011, 8, 30, 10, 0, 0)
      , end = new Date(2011, 9, 1, 11, 0, 0);

    var seedEv = getSeedEvent (start, end);
    seedEv.frequency = 'week';

    seedEv.recurrence.every = 2;
    seedEv.recurrence['end-by'].after = 20;
    seedEv.recurrence['repeat-on'] = { f: 'on', t: 'on', m: 'on' };

    var Seed = Carilu.seed(seedEv, {
            start: start
          , end: end
        });

    // limit into empty date
    Seed.setPeriod(new Date(2011, 9, 8, 2, 18));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    // limit date into first instance
    Seed.setPeriod(new Date(2011, 8, 30, 22, 30));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    // limit date previous first instance
    Seed.setPeriod(new Date(2011, 8, 30));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    // limit date equal end date first instance
    Seed.setPeriod(new Date(2011, 9, 1, 11, 0, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 11, 11, 0, -1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 11, 11, 0, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 12, 11, 0, 0));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(3, elapsedInstances);

    Seed.setPeriod(new Date(2011, 11, 10));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(15, elapsedInstances);

    Seed.setPeriod(new Date(2011, 11, 10, 11));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(16, elapsedInstances);

    Seed.setPeriod(new Date(2012, 0, 29));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(25, elapsedInstances);


    seedEv.allDay = true;
    // re-normalize event
    Seed.normalize();

    Seed.setPeriod(new Date(2011, 11, 10, 11));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(15, elapsedInstances);
  },

  'getElapsedInstances() - monthly - day of month': function() {
    _print ('Seed\t', 'getElapsedInstances() - monthly - day of month');

    seedMonthlyEvent.recurrence['repeat-on'] = { mode: 'day-of-month' };

    // seed object
    var Seed = Carilu.seed(seedMonthlyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    Seed.setPeriod(new Date(2011, 8, 25));
    var elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 1, 22, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 10, 29, 1, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 10, 29, 1, 0, 1));
    elapsedInstances = Seed.getElapsedInstances(null, true);
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2011, 10, 29, 16));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2012, 5, 27));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(5, elapsedInstances);
  },

  'getElapsedInstances() - monthly - day of week': function() {
    _print ('Seed\t', 'getElapsedInstances() - monthly - day of week');

    var start = new Date(2011, 8, 26, 15, 0, 0)
      , end = new Date(2011, 8, 29, 16, 0, 0);

    seedMonthlyEvent.recurrence['repeat-on'] = { mode: 'day-of-week' };

    // seed object
    var Seed = Carilu.seed(seedMonthlyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    Seed.setPeriod(new Date(2011, 8, 25));
    var elapsedInstances = Seed.getElapsedInstances();
    assert.equal(0, elapsedInstances);

    Seed.setPeriod(new Date(2011, 9, 1, 22, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 10, 28, 22, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 10, 29, 22, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);


    Seed.setPeriod(new Date(2011, 11, 1, 15, 59, 59));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(1, elapsedInstances);

    Seed.setPeriod(new Date(2011, 11, 1, 11));
    elapsedInstances = Seed.getElapsedInstances(null, true);
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2011, 11, 1, 16, 0, 0, 1));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(2, elapsedInstances);

    Seed.setPeriod(new Date(2012, 5, 27));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(5, elapsedInstances);

    Seed.setPeriod(new Date(2012, 6, 27));
    elapsedInstances = Seed.getElapsedInstances();
    assert.equal(6, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 0, 26));
    assert.equal(2, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 0, 26), true);
    assert.equal(3, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 0, 26, 16));
    assert.equal(3, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 0, 26, 15, 59, 59));
    assert.equal(2, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 6, 29));
    assert.equal(6, elapsedInstances);


    seedMonthlyEvent.recurrence.every = 1;
    // seed object
    Seed = Carilu.seed(seedMonthlyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    elapsedInstances = Seed.getElapsedInstances(new Date(2011, 10, 29));
    assert.equal(2, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2011, 10, 29), true)
    assert.equal(3, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 0, 8))
    assert.equal(4, elapsedInstances);
  },

  'getElapsedInstances() - yearly': function() {
    _print ('Seed\t', '_getDateNextInstance() - yearly');
    // seed object
    var Seed = Carilu.seed(seedYearlyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var elapsedInstances = Seed.getElapsedInstances(new Date(2011, 8, 25));
    assert.equal(0, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2012, 6, 29));
    assert.equal(1, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2014, 6, 29));
    assert.equal(2, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2015, 8, 29));
    assert.equal(2, elapsedInstances);

    elapsedInstances = Seed.getElapsedInstances(new Date(2015, 8, 30));
    assert.equal(3, elapsedInstances);
  },

  '_getJumpingVector - weekly': function() {

    // Oct 2011
    //
    //   S    M    T    W   Th    F   Sa
    //  ---------------------------------
    //  25   26   27   28   29   30    1
    //   2    3    4    5    6    7    8
    //   9   10   11   12   13   14   15
    //  16   17   18   19   20   21   22
    //  23   24   25   26   27   28   29
    //  30   31    1    2    3    4    5

    _print ('Seed\t', 'jumpingVector - weekly');

    function compare (vec1, vec2) {
      for (var i = 0; i < vec1.length; i++) {
        assert.equal(vec1[i], vec2[i], vec1 + ' == ' + vec2)
      };
    }

    var Seed = Carilu.seed(seedWeeklyEvent);

    var vector = Seed._getJumpingVector()
      , compareTo = [null, 1, 3, null, null, 10, null];
    compare(vector, compareTo);

    Seed.ev.recurrence['repeat-on'] = { m: 'on' };
    vector = Seed.setJumpingVector();
    compareTo = [null, 14, null, null, null, null, null];
    compare(vector, compareTo);
  },

  'getInstances() - normal event': function() {
    _print ('Seed\t', 'getInstances() - normal event');

    var Seed = Carilu.seed(normalEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
          , addNoRec: true
        });

    Seed.setPeriod(start, end);
    var instances = Seed.getInstances();
    assert.equal(instances.length, 1);

    Seed.setPeriod(new Date(2011, 8, 1));
    instances = Seed.getInstances();
    assert.equal(instances.length, 1);

    Seed.setPeriod(new Date(2011, 8, 26, 10, 0, 0), new Date(2011, 8, 30));
    instances = Seed.getInstances();
    assert.equal(instances.length, 1);

    Seed.setPeriod(new Date(2011, 8, 28, 10, 0, 0), new Date(2011, 8, 30));
    instances = Seed.getInstances();
    assert.equal(instances.length, 0);
  },

  'getInstances() - daily - never': function() {
    _print ('Seed\t', 'getInstances() - daily - never');

    seedDailyEvent.recurrence['end-by'].type = 'never';

    var Seed = Carilu.seed(seedDailyEvent, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 9);

    Seed.setPeriod(new Date(2011, 9, 30), new Date(2011, 11, 11));
    instances = Seed.getInstances();
    assert.equal(instances.length, 9);

    Seed.setPeriod(new Date(2011, 8, 26), new Date(2011, 11, 11));
    instances = Seed.getInstances();
    assert.equal(instances.length, 16);
  },

  'getInstances() - daily - after': function() {
    _print ('Seed\t', 'getInstances() - daily - after');

    var start = new Date(2011, 8, 26, 10, 0, 0)
      , end = new Date(2011, 8, 27, 11, 0, 0);

    // period
    var startDate = new Date (2011, 8, 25)
      , endDate = new Date (2011, 10, 6);

    var seedEv = getSeedEvent (start, end);

    var Seed = Carilu.seed(seedEv, {
            start: startDate
          , end: endDate
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 4);

    // stopped by end period date (9 instances)
    seedEv.recurrence.every = 5;
    seedEv.recurrence['end-by'].after = 220;
    instances = Seed.getInstances();
    assert.equal(instances.length, 9);

    // stopped by end period date (2 instances)
    Seed.setPeriod(null, new Date(2011, 9, 1, 10, 1 ,0));
    instances = Seed.getInstances();
    assert.equal(instances.length, 2);

    // stopped by end period date (0 instance)
    Seed.setPeriod(null, new Date(2011, 8, 8));
    instances = Seed.getInstances();
    assert.equal(instances.length, 0);
  },

  'getInstances() - daily - on': function() {
    _print ('Seed\t', 'getInstances() - daily - on');

    var start = new Date(2011, 8, 26, 10, 0, 0)
      , end = new Date(2011, 8, 27, 11, 0, 0);

    // period
    var startDate = new Date (2011, 8, 25)
      , endDate = new Date (2011, 10, 6);

    var seedEv = getSeedEvent (start, end);
    seedEv.recurrence['end-by'].type = 'on';
    seedEv.recurrence['end-by'].on = new Date (2011, 1, 1);

    var Seed = Carilu.seed(seedEv, {
            start: startDate
          , end: endDate
        });

    instances = Seed.getInstances();
    assert.equal(instances.length, 0);

    Seed.ev.recurrence['end-by'].on = new Date (2021, 10, 1);
    instances = Seed.getInstances();
    assert.equal(instances.length, 11);

    Seed.ev.recurrence['end-by'].on = new Date (2011, 9, 11);
    instances = Seed.getInstances();
    assert.equal(instances.length, 4);

  },

  'getInstances() - weekly': function() {
    _print ('Seed\t', 'getInstances() - weeekly');

    seedWeeklyEvent.recurrence['end-by'].after = 100;
    seedWeeklyEvent.recurrence['repeat-on'] = { f: 'on', t: 'on', m: 'on' };

    var Seed = Carilu.seed(seedWeeklyEvent, {
            start:  new Date (2011, 8, 25)
          , end: new Date (2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 7);

    Seed.setPeriod(new Date(2011, 9, 11), new Date(2011, 9, 12));
    instances = Seed.getInstances();
    assert.equal(instances.length, 2);

    Seed.setPeriod(new Date(2012, 3, 5), new Date(2012, 4, 1));
    instances = Seed.getInstances();
    assert.equal(instances.length, 6);

    // move first series date
    seedWeeklyEvent.recurrence['repeat-on'] = { t: 'on', m: 'on' };
    var Seed = Carilu.seed(seedWeeklyEvent, {
            start:  new Date (2011, 8, 25)
          , end: new Date (2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 6);
  },

   'getInstances() - monthly - day of month': function() {
    _print ('Seed\t', 'getInstances() - monthly - day of month');

    var start = new Date(2011, 8, 26, 15, 0, 0)
      , end = new Date(2011, 8, 29, 16, 0, 0);

    var seedEv = getSeedEvent(start, end);
    seedEv.frequency = 'month';
    seedEv.recurrence.every = 1;
    seedEv.recurrence['end-by'].type = 'never';
    seedEv.recurrence['repeat-on'] = { mode: 'day-of-month' };

    // seed object
    var Seed = Carilu.seed(seedEv, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 2);

    Seed.setPeriod(new Date(2011, 9, 30), new Date(2011, 11, 11));
    instances = Seed.getInstances();
    assert.length(instances, 1);

    Seed.setPeriod(new Date(2011, 10, 27), new Date(2012, 0, 8));
    instances = Seed.getInstances();
    assert.length(instances, 2);

    Seed.setPeriod(new Date(2012, 0, 1), new Date(2012, 1, 11));
    instances = Seed.getInstances();
    assert.length(instances, 1);
  },


   'getInstances() - monthly - day of week': function() {
    _print ('Seed\t', 'getInstances() - monthly - day of week');

    var start = new Date(2011, 8, 26, 15, 0, 0)
      , end = new Date(2011, 8, 29, 16, 0, 0);

    var seedEv = getSeedEvent(start, end);
    seedEv.frequency = 'month';
    seedEv.recurrence.every = 1;
    seedEv.recurrence['end-by'].type = 'never';
    seedEv.recurrence['repeat-on'] = { mode: 'day-of-week' };

    // seed object
    var Seed = Carilu.seed(seedEv, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 2);

    Seed.setPeriod(new Date(2011, 9, 30), new Date(2011, 11, 11));
    instances = Seed.getInstances();
    assert.length(instances, 1);

    Seed.setPeriod(new Date(2011, 10, 27), new Date(2012, 0, 8));
    instances = Seed.getInstances();
    assert.length(instances, 2);

    Seed.setPeriod(new Date(2012, 0, 1), new Date(2012, 1, 11));
    instances = Seed.getInstances();
    assert.length(instances, 1);

    Seed.setPeriod(new Date(2012, 0, 1), new Date(2012, 1, 27, 14, 59, 59, 999));
    instances = Seed.getInstances();
    assert.length(instances, 2);

    Seed.setPeriod(new Date(2012, 0, 1), new Date(2012, 1, 27, 14, 59, 59, 1000));
    instances = Seed.getInstances();
    assert.length(instances, 2);
  },

   'getInstances() - yearly': function() {
    _print ('Seed\t', 'getInstances() - yearly');

    var start = new Date(2011, 8, 26, 15, 0, 0)
      , end = new Date(2011, 8, 29, 16, 0, 0);

    var seedEv = getSeedEvent(start, end);
    seedEv.frequency = 'year';
    seedEv.recurrence.every = 1;

    // seed object
    var Seed = Carilu.seed(seedEv, {
            start: new Date(2011, 8, 25)
          , end: new Date(2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 1);

    Seed.setPeriod(new Date(2011, 9, 30), new Date(2011, 11, 11));
    instances = Seed.getInstances();
    assert.length(instances, 0);

    Seed.setPeriod(new Date(2010, 0, 1), new Date(2040, 0, 1));

    instances = Seed.getInstances();
    assert.length(instances, 4);

    Seed.ev.recurrence['end-by'].type = 'never';
    instances = Seed.getInstances();
    assert.length(instances, 29);
  },

  'Instance.getNumber() - daily - never - couting': function() {
    _print ('Seed\t', 'getInstances() - daily - never');

    seedDailyEvent.recurrence['end-by'].type = 'never';
    var Seed = Carilu.seed(seedDailyEvent, {
            start: new Date(2011, 9, 4)
          , end: new Date(2011, 10, 6)
        });

    var instances = Seed.getInstances();
    assert.equal(instances.length, 7);

    for (var i = 0; i < instances.length; i++) {
      var inst = instances[i];
      assert.equal(instances[i].getNumber(), i + 2);
    };
  },

  'Instance.getPrev() - daily - never': function() {
    _print ('Seed\t', 'getPrev() - daily');

    seedDailyEvent.recurrence['end-by'].type = 'never';
    var Seed = Carilu.seed(seedDailyEvent, {
            start: new Date(2011, 9, 4)
          , end: new Date(2011, 10, 6)
        });

    var instances = Seed.getInstances()
      , inst = instances[3]
      , prev = inst.getPrev();

      assert.equal(prev.start.toGMTString(), 'Sun, 16 Oct 2011 00:00:00 GMT');
  },


};
