
/**
 * Module dependencies.
 */

var Util = require('../lib/util')
  , assert = require('assert')

/**
 * Alias
 */

var _date = Util.date
  , event = Util.event;

var _print = require('./print');

  /**
   * test usefull function
   * return a event seed
   *
   * @param {Date} start
   * @para, {Date} end
   */

  function getSeedEvent (start, end) {
    return seedEv = {
        title: 'getRecEvents'
      , start: start
      , end: end
      , allDay: false
      , frequency: 'day'
      , recurrence: {
            'end-by': {
                after: 4
              , type: 'after'
            }
          , every: 4
        }
    }
  }

  module.exports = {

  'getFormattedDate() - complete with zero': function() {
    _print ('util - date', 'getFormattedDate() - complete with zero');

    var start = new Date (1977, 6, 8, 16, 40, 35);

    // start date: Sat, 01 Oct 2011 13:30:00 GMT
    var date_string = _date.getFormattedDate(start, true);
    assert.equal(date_string, '07/08/1977');
  },

  'getFormattedDate() - no complete': function() {
    _print ('util - date', 'getFormattedDate() - no complete');

    var start = new Date (1977, 6, 8, 16, 40, 35)
      , date_string = _date.getFormattedDate(start);

    assert.equal(date_string, '7/8/1977');
  },

  'getDiff()': function() {
    _print ('util - date', 'getDiff');

    var start = new Date (2011, 10, 1, 16);

    for (var i = 1; i < 1000; i++) {
      var limit = new Date(start);
      limit.setDate(i);

      var diffInDays = _date.getDiff(limit, start) / _date.const.day;
      assert.equal(0, (diffInDays % 1));
    };
  },

  'getDiffInDays() - round': function() {
    _print ('util - date', 'getDiffInDays() - round');

    var start = new Date (2011, 10, 1, 16)
      , end = new Date (2011, 10, 9, 16);
    assert.equal(8, _date.getDiffInDays(end, start));

    start = new Date (2011, 10, 1, 16);
    end = new Date (2011, 10, 9, 22);
    assert.equal(9, _date.getDiffInDays(end, start));
  },

  'getDiffInMonths() - round': function() {
    _print ('util - date', 'getDiffInMonths() - round');

    var start = new Date (2011, 8, 26, 16, 40, 35)
      , end = new Date (2011, 8, 28);
    assert.equal(0, _date.getDiffInMonths(end, start));

    end = new Date (2011, 11, 10);
    assert.equal(3, _date.getDiffInMonths(end, start));

    end = new Date (2012, 4, 30);
    assert.equal(8, _date.getDiffInMonths(end, start));
  },

  'getDiffInMonths() - NOT round': function() {
    _print ('util - date', 'getDiffInMonths() - NOT round');

    var start = new Date (2011, 8, 26, 16, 40, 35)
      , end = new Date (2011, 8, 28);
    assert.equal(0, _date.getDiffInMonths(end, start, true));

    end = new Date (2011, 11, 10);
    assert.equal(2, _date.getDiffInMonths(end, start, true));

    end = new Date (2012, 4, 30);
    assert.equal(8, _date.getDiffInMonths(end, start, true));

    end = new Date (2011, 9, 26, 16, 40, 35)
    assert.equal(1, _date.getDiffInMonths(end, start));
    assert.equal(1, _date.getDiffInMonths(end, start, true));

    end = new Date (2011, 9, 26, 16, 40, 34)
    assert.equal(1, _date.getDiffInMonths(end, start));
    assert.equal(0, _date.getDiffInMonths(end, start, true));

  },

  'getFormattedTime()': function() {
    _print ('util - date', 'getFormattedTime()');

    var time_string = _date.getFormattedTime(new Date (1977, 6, 8, 1, 40, 35));
    assert.equal(time_string, '01:40 AM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 0, 40, 35));
    assert.equal(time_string, '12:40 AM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 11, 40, 35));
    assert.equal(time_string, '11:40 AM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 12, 40, 35));
    assert.equal(time_string, '12:40 PM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 13, 40, 35));
    assert.equal(time_string, '01:40 PM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 19, 40, 35));
    assert.equal(time_string, '07:40 PM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 23, 40, 35));
    assert.equal(time_string, '11:40 PM');

    time_string = _date.getFormattedTime(new Date (1977, 6, 8, 24, 40, 35));
    assert.equal(time_string, '12:40 AM');
  },


  'parseISO8601()': function() {
    _print ('util - date', 'parseISO8601()');

    var str_date= '1977-07-18T20:40:35.012Z'
      , date = new Date(1977, 6, 18, 20, 40, 35, 12);

    assert.equal(_date.parseISO8601(str_date).toUTCString(), date.toUTCString());
  },

  'parseFromWeirdISOFormat() - \'yyyyddmm\' format': function (str) {
    _print('util - date', 'parseDate: fromWeirdISOFormat - \'yyyyddmm\' format');

    // test date: Mon, 18 Jul 1977 00:00:00 GMT
    var strDate = '19770718'
      , date = new Date(1977, 6, 18)
      , wiredDate = _date.parseFromWeirdISOFormat(strDate);

    assert.equal(+date, +wiredDate);
  },

  'parseFromWeirdISOFormat() - \'yyyyddmmThhmmssZ\' format': function (str) {
    _print('util - date'
        , 'parseDate: fromWeirdISOFormat - \'yyyyddmmThhmmssZ\' format');

    // test date: Mon, 18 Jul 1977 18:35:54 GMT
    var strDate = '19770718T183554Z'
      , date = new Date(1977, 6, 18, 18, 35, 54)
      , wiredDate = _date.parseFromWeirdISOFormat(strDate);
    assert.equal(+date, +wiredDate);
  },

  'clearTime() - not clone': function() {
    _print ('util - date', 'clearTime() - not clone');

    var oct2011 = new Date(2011, 9 , 1, 14, 32, 12, 987);
    _date.clearTime (oct2011);

    assert.equal(oct2011.toGMTString(), 'Sat, 01 Oct 2011 00:00:00 GMT');
  },

  'clearTime() - clone': function() {
    _print ('util - date', 'clearTime() - clone');

    var oct2011 = new Date(2011, 9 , 1, 14, 32, 12, 987);
    var newCleartedDate = _date.clearTime (oct2011, true);

    assert.equal(oct2011.toGMTString(), 'Sat, 01 Oct 2011 14:32:12 GMT');
    assert.equal(newCleartedDate.toGMTString(), 'Sat, 01 Oct 2011 00:00:00 GMT');
  },

  'isLeapYear()': function(str) {
    _print ('util - date', 'isLeapYear()');
    var leaps = [1976, 1980, 1984, 1988, 1992, 1996, 2000, 2004, 2008, 2012, 2016];
    var oct2011 = new Date(1976, 6 , 18, 23, 48, 54);

    for (var year = 1976; year < 2016; year++) {
      assert.equal(leaps.indexOf(year) >= 0, _date.isLeapYear(year)
          , year + ' not is leap');
    };

  },

  'clearTime() - clone': function() {
    _print ('util - date', 'clearTime() - clone');

    var oct2011 = new Date(2011, 9 , 1, 14, 32, 12, 987);
    var newCleartedDate = _date.clearTime (oct2011, true);

    assert.equal(oct2011.toGMTString(), 'Sat, 01 Oct 2011 14:32:12 GMT');
    assert.equal(newCleartedDate.toGMTString(), 'Sat, 01 Oct 2011 00:00:00 GMT');
  },



  'setByWeekAndDay()': function() {
    _print ('util - date', 'setByWeekAndDay()');

    var date = new Date(2011, 9, 1);
    _date.setByWeekAndDay (date, 2, 4);
    assert.equal(date.toUTCString(), 'Thu, 20 Oct 2011 00:00:00 GMT');

    _date.setByWeekAndDay (date, 0, 1);
    assert.equal(date.toUTCString(), 'Mon, 03 Oct 2011 00:00:00 GMT');

    _date.setByWeekAndDay (date, 0, 5);
    assert.equal(date.toUTCString(), 'Fri, 07 Oct 2011 00:00:00 GMT');

    var date = new Date(2011, 5, 1);
    _date.setByWeekAndDay (date, 0, 2);
    assert.equal(date.toUTCString(), 'Tue, 07 Jun 2011 00:00:00 GMT');

  },

  'getWeekOfDay()': function() {
    _print ('util - date', 'getWeekOfDay()');

    var today = new Date
      , oct2011 = new Date(2011, 9 , 1
          , today.getHours()
          , today.getMilliseconds()
          , today.getSeconds()
          , today.getMinutes()
        )

    var weekNumberVector =  [               null, 0
                              , 0, 0, 0, 0, 0, 0, 1
                              , 1, 1, 1, 1, 1, 1, 2
                              , 2, 2, 2, 2, 2, 2, 3
                              , 3, 3, 3, 3, 3, 3, 4
                              , 4, 4, 4, 4, 4, 4, 5
                            ];


    // walk all days of Oct 2011
    var w;
    for (var i = 1; i <= 31; i++) {
      oct2011.setDate(i);
      w = _date.getWeekOfDay(oct2011);
      assert.equal(weekNumberVector[i], w
        , oct2011.toUTCString() + ' | ' + weekNumberVector[i] + ' == ' + w);

    };

    var may2012 = new Date (oct2011);
    may2012.setMonth(4);
    may2012.setFullYear(2012);

    var weekNumberVector =  [   null, 0, 0, 0, 0, 0
                              , 0, 0, 1, 1, 1, 1, 1
                              , 1, 1, 2, 2, 2, 2, 2
                              , 2, 2, 3, 3, 3, 3, 3
                              , 3, 3, 4, 4, 4, 4, 4
                            ];

    // walk all days of Oct 2011
    for (var i = 1; i <= 31; i++) {
      may2012.setDate(i);
      w = _date.getWeekOfDay(may2012);
      assert.equal(weekNumberVector[i], w
        , may2012.toUTCString() + ' | ' + weekNumberVector[i] + ' == ' + w);
    };

  },
/*
  'getDuration()': function() {
    _print ('util - event', 'getDuration()');

    //    12  -  13  -  14
    var d1 = new Date (2011, 9, 12, 21, 43, 54)
      , d2 = new Date (2011, 9, 14, 21, 28, 17);

    var ev1 = {
            title: 'Event 1'
          , start: d1
          , end: d2
        };

    var duration = null;

    // duration for ev1, in days -> 2
    duration = event.getDuration(ev1);
    // value text
    assert.equal(duration.val, 3);
    // rest test
    assert.equal(duration.rest > 0, false, 'rest: ' + duration.rest);


    // duration for ev1, in days -> 1 (floor method)
    duration = event.getDuration(ev1, null, 'floor');
    // value text
    assert.equal(duration.val, 2);
    // rest test
    assert.equal(duration.rest > 0, true, 'rest: ' + duration.rest);


    // duration for ev1, in hpures -> 48
    duration = event.getDuration(ev1, 'hour');
    // value text
    assert.equal(duration.val, 48);
    // rest test
    assert.equal(duration.rest > 0, false, 'rest: ' + duration.rest);


    // duration for ev1, in hours -> 47 (floor method)
    duration = event.getDuration(ev1, 'hour', 'floor');
    // value text
    assert.equal(duration.val, 47);
    // rest test
    assert.equal(duration.rest > 0, true, 'rest: ' + duration.rest);


    ev1.start = new Date(2011, 8, 26, 10, 0, 0);
    ev1.end = new Date(2011, 8, 26, 12, 0, 0);
    // duration for ev1, in hours -> 47 (floor method)
    duration = event.getDuration(ev1, 'day');
    // value text
    assert.equal(duration.val, 1);


    ev1.end = new Date(2011, 8, 29, 23, 59, 59);
    // duration for ev1, in hours -> 47 (floor method)
    duration = event.getDuration(ev1, 'day');
    // value text
    assert.equal(duration.val, 4);


    ev1.end = new Date(2011, 8, 29, 0, 0, 0);
    // duration for ev1, in hours -> 47 (floor method)
    duration = event.getDuration(ev1, 'day');
    // value text
    assert.equal(duration.val, 4);
  },
  */
};
